""" Descriptor widgets
"""

from __future__ import unicode_literals
from urwid import Text
from functools import partial


# Show the user this view can be scrolled
MetaScroll = partial(Text, "(\u21C5 Scroll)")



""" Represents a Single Juju machine UI widget
"""

from urwid import Text


class MachineWidget:

    def __init__(self, machine):
        """ Init

        Params:
        machine: Juju Machine Dictionary
        """
        self.machine = machine
        _attrs = ['Id',
                  'Hardware',
                  'Series',
                  'WantsVote',
                  'InstanceState',
                  'DNSName',
                  'HasVote',
                  'InstanceID']
        for m in _attrs:
            setattr(self, m, Text(str(self.machine.get(m))))



from urwid import Text


class Instruction(Text):

    def __init__(self, text, **kwargs):
        super().__init__(('info_context', text), **kwargs)


class ColumnHeader(Text):

    def __init__(self, text, **kwargs):
        super().__init__(('info_context', text), **kwargs)



from __future__ import unicode_literals
from urwid import Button, Text
from functools import partial


class PlainButton(Button):
    button_left = Text("[")
    button_right = Text("]")


class MenuSelectButton(Button):
    button_left = Text("")
    button_right = Text("")


confirm_btn = partial(PlainButton, label="Confirm", on_press=None)
submit_btn = partial(PlainButton, label="Submit", on_press=None)
cancel_btn = partial(PlainButton, label="Cancel", on_press=None)
back_btn = partial(PlainButton, label="Back", on_press=None)
quit_btn = partial(PlainButton, label="Quit", on_press=None)
done_btn = partial(PlainButton, label="Done", on_press=None)
reset_btn = partial(PlainButton, label="Reset", on_press=None)
menu_btn = partial(MenuSelectButton, on_press=None)
select_btn = partial(MenuSelectButton,
                     label="Select \u2193",
                     on_press=None)



from __future__ import unicode_literals

from urwid import (Columns, ListBox)
from ubuntui.widgets.hr import HR


class Table:

    def __init__(self):
        self._rows = []
        self._row_id = []
        self._is_header_set = False

    def addHeadings(self, headings):
        """ Takes list of headings and converts them to column header
        with appropriate color

        Params:
        headings: List of column text headings
        """
        if not self._is_header_set:
            self.addRow(Columns(headings), False)
            self._is_header_set = True

    def addColumns(self, row_id, columns, dividechars=1, force=False):
        """ Convert list of widgets to Columns and add to a table row

        Arguments:
        row_id: unique id of new row
        columns: list of columns
        dividechars: character space between columns
        force: force additional row regardless of existing row_id
        """
        if row_id not in self._row_id or force:
            if row_id not in self._row_id:
                self._row_id.append(row_id)

            # If we force an additional row it's usually to expand
            # on the previous row so we do not display a divider
            # between the two.
            use_divider = True
            if force:
                use_divider = False
            self.addRow(Columns(columns, dividechars=dividechars), use_divider)

    def addRow(self, item, use_divider=True):
        """ Appends widget to Pile

        Arguments:
        item: Widget to add to listbox
        use_divider: use divider for row item
        """
        if use_divider and len(self._rows) != 0:
            self._rows.append(HR(0, 0))
        self._rows.append(item)

    def render(self):
        return ListBox(self._rows)



""" Horizontal rule widget
"""

from __future__ import unicode_literals
from urwid import Divider
from ubuntui.utils import Color


# Show the user this view can be scrolled
def HR(top=1, bottom=1):
    """ Returns a horiztonal divider

    Arguments:
    top: padding top
    bottom: padding bottom
    """
    return Color.divider_line(
        Divider("\N{BOX DRAWINGS LIGHT HORIZONTAL}", top, bottom))



from urwid import Pile, Text, WidgetWrap

from .lists import SimpleList
from .utils import Padding, Color


class Header(WidgetWrap):
    """ Header Widget

    This widget uses the style key `frame_header`

    Arguments:
    title: Title text
    excerpt: Additional header text
    align: Text alignment, defaults=left
    """

    def __init__(self, title=None, excerpt=None, align="left"):
        self._title = title
        self._excerpt = excerpt
        widgets = []
        if self._title is not None:
            widgets.append(Color.frame_header(Padding.line_break("")))
            widgets.append(Color.frame_header(
                Padding.left(Text(self._title.upper()), left=2)))
            widgets.append(Color.frame_header(Padding.line_break("")))
        if self._excerpt is not None:
            widgets.append(Text(""))
            widgets.append(
                Padding.center_90(
                    Text(("body", self._excerpt))))
        widgets.append(Text(""))
        super().__init__(Pile(widgets))

    @property
    def title(self):
        return self._title.get_text()[0]

    @title.setter
    def title(self, val, attr=None):
        """
        Sets header title text

        Arguments:
        val: Text value
        attr: (optional) Attribute lookup
        """
        if attr is not None:
            self._title.set_text(val.upper())
        else:
            self._title.set_text((attr, val.upper()))


class Footer(WidgetWrap):
    """ Footer widget
    Style key: `frame_footer`
    """

    def __init__(self, message="", completion=0):
        message_widget = (Text(message, align="center"))
        status = [
            message_widget
        ]
        super().__init__(Color.frame_footer(Pile(status)))


class Body(WidgetWrap):
    """ Body widget
    """

    def __init__(self):
        text = [
            Padding.line_break(""),
        ]
        w = (SimpleList(text))
        super().__init__(w)



from collections import OrderedDict
from urwid import (WidgetWrap, signals, emit_signal, connect_signal,
                   RadioButton, Button, Columns, Text, Pile,
                   Divider, Filler)
from ubuntui.utils import Color, Padding


class SelectorWidget(WidgetWrap):

    """
    Simple selector box
    """
    __metaclass__ = signals.MetaSignals
    signals = ['done']

    def __init__(self, title, opts, cb):
        """ Init

        Arguments:
        title: Title of widget
        opts: List of string options
        cb: callback function
        """
        self.title = title
        self.radio_items = OrderedDict()
        for item in opts:
            self.add_radio(item)
        connect_signal(self, 'done', cb)
        super().__init__(self._build_widget())

    def add_radio(self, item, group=[]):
        self.radio_items[item] = RadioButton(group, item)

    def _build_buttons(self):
        buttons = [
            Color.button_primary(
                Button("Confirm", self.submit),
                focus_map='button_primary focus'),
            Color.button_secondary(
                Button("Cancel", self.cancel),
                focus_map='button_secondary focus')
        ]
        return Pile(buttons)

    def _build_widget(self):
        total_items = [
            Padding.center_60(Text(self.title, align="center")),
            Padding.center_60(
                Divider("\N{BOX DRAWINGS LIGHT HORIZONTAL}", 1, 1))
        ]
        for item in self.radio_items.keys():
            opt = self.radio_items[item]
            col = Columns([opt])
            total_items.append(Padding.center_60(col))
        total_items.append(
            Padding.center_60(
                Divider("\N{BOX DRAWINGS LIGHT HORIZONTAL}", 1, 1)))
        total_items.append(Padding.center_20(self._build_buttons()))
        return Filler(Pile(total_items), valign='middle')

    def submit(self, button):
        for item in self.radio_items.keys():
            _item = self.radio_items[item]
            if _item.get_state():
                selected_item = _item.label
        self.emit_done_signal(selected_item)

    def cancel(self, button):
        raise SystemExit("Installation cancelled.")

    def emit_done_signal(self, *args):
        emit_signal(self, 'done', *args)


class SelectorWithDescriptionWidget(WidgetWrap):

    """
    Simple selector box

    :param str title: title of selections
    :param list opts: items to select ('name', 'description')
    :param cb: callback
    :returns: item selected from dialog
    """
    __metaclass__ = signals.MetaSignals
    signals = ['done']

    def __init__(self, title, opts, cb):
        self.title = title
        self.radio_items = OrderedDict()
        for item, desc in opts:
            self.add_radio(item, desc)
        connect_signal(self, 'done', cb)
        super().__init__(self._build_widget())

    def add_radio(self, item, desc, group=[]):
        self.radio_items[item] = (RadioButton(group, item), desc)

    def _build_buttons(self):
        buttons = [
            Color.button_primary(
                Button("Confirm", self.submit),
                focus_map='button_primary focus'),
            Color.button_secondary(
                Button("Cancel", self.cancel),
                focus_map='button_secondary focus')
        ]
        return Pile(buttons)

    def _build_widget(self):
        total_items = [
            Padding.center_60(Text(self.title, align="center")),
            Padding.center_60(
                Divider("\N{BOX DRAWINGS LIGHT HORIZONTAL}", 1, 1))
        ]
        for item in self.radio_items.keys():
            opt, desc = self.radio_items[item]
            col = Columns(
                [
                    ("weight", 0.4, opt),
                    Text(desc)
                ], dividechars=1)
            total_items.append(Padding.center_60(col))
        total_items.append(
            Padding.center_60(
                Divider("\N{BOX DRAWINGS LIGHT HORIZONTAL}", 1, 1)))
        total_items.append(Padding.center_20(self._build_buttons()))
        return Filler(Pile(total_items), valign='middle')

    def submit(self, button):
        for item in self.radio_items.keys():
            _item = self.radio_items[item][0]
            if _item.get_state():
                selected_item = _item.label
        self.emit_done_signal(selected_item)

    def cancel(self, button):
        raise SystemExit("Installation cancelled.")

    def emit_done_signal(self, *args):
        emit_signal(self, 'done', *args)



""" Represents a Single MAAS node UI widget
"""

from urwid import WidgetWrap, Text


class MachineWidget(WidgetWrap):

    def __init__(self, machine):
        """ Init

        Params:
        machine: Maas Machine class
        """
        self.machine = machine
        _attrs = ['hostname',
                  'status',
                  'zone',
                  'cpu_count',
                  'storage',
                  'architecture',
                  'memory',
                  'power_type',
                  'power_state',
                  'system_id',
                  'ip_addresses',
                  'macaddress_set',
                  'tag_names',
                  'owner']
        for m in _attrs:
            if isinstance(m, list):
                continue
            setattr(self, m, Text(str(self.machine.get(m))))



""" Palette Loader """

# Palette
orange = "#f60"
light_orange = "#f80"
dark_magenta = "#608"
light_magenta = "#f0f"
light_red = "#f00"
light_green = "#0d0"
white = "white"
black = "black"
light_gray = "g82"
cool_gray = "g50"
warm_gray = "g15"
blue = "#08f"
dark_blue = "#00f"


# Styles
STYLES = [
    ("frame_header", "", "", "", white, orange),
    ("frame_subheader", "", "", "", white, warm_gray),
    ("frame_excerpt", "", "", "", cool_gray, ""),
    ("frame_footer", "", "", "", white, warm_gray),
    ("body", "", "", "", "", ""),
    ("button_primary", "", "", "", "", ""),
    ("button_primary focus", "", "", "", white, light_green),
    ("button_secondary", "", "", "", "", ""),
    ("button_secondary focus", "", "", "", white, cool_gray),
    ("menu_button", "", "", "", white, cool_gray),
    ("menu_button focus", "", "", "", white, cool_gray),
    ("info_minor", "", "", "", warm_gray, ""),
    ("info_major", "", "", "", light_green, ""),
    ("info_context", "", "", "", cool_gray, ""),
    ("error_major", "", "", "", light_red, ""),
    ("status_info", "", "", "", light_green, warm_gray),
    ("status_error", "", "", "", light_red, warm_gray),
    ("string_input", "", "", "", white, cool_gray),
    ("string_input focus", "", "", "", white, orange),
    ("dialog", "", "", "", white, warm_gray),
    ("radio_input", "", "", "", white, warm_gray),
    ("radio_input focus", "", "", "", orange, warm_gray),
    ("header_title", "", "", "", orange, ""),
    ("pending_icon_on", "", "", "", blue, ""),
    ("pending_icon", "", "", "", blue, ""),
    ("error_icon", "", "", "", light_red, ""),
    ("success_icon", "", "", "", light_green, ""),
    ("column_header", "", "", "", white, blue),
    ('deploy_highlight_start', "", "", "", warm_gray, light_green),
    ('deploy_highlight_end', "", "", "", warm_gray, cool_gray),
    ('disabled_button', "", "", "", light_gray, warm_gray),
    ('disabled_button_focus', "", "", "", light_gray, warm_gray),
    ('divider_line', "", "", "", cool_gray, ""),
    ('filter', "", "", "", black, light_gray),
    ('filter_focus', "", "", "", white, warm_gray),
    ('focus', "", "", "", white, warm_gray),
    ('radio focus', "", "", "", white, dark_magenta),
    ('status_extra', "", "", "", light_gray, warm_gray),
    ('error', "", "", "", white, light_red),
    ('info', "", "", "", light_green, ''),
    ('label', "", "", "", light_gray, ''),
]



import urwid
import asyncio
import uuid
import logging

log = logging.getLogger('eventloop')


class EventLoopException(Exception):
    """ Problem with eventloop
    """
    pass


class EventLoop:

    """ Abstracts out event loop
    """
    loop = None
    alarms = {}
    extra_opts = {
        'screen': urwid.raw_display.Screen(),
        'handle_mouse': True
    }

    @classmethod
    def build_loop(cls, ui, palette, **kwargs):
        """ Builds eventloop
        """
        cls.extra_opts['screen'].set_terminal_properties(colors=256)
        cls.extra_opts.update(**kwargs)
        evl = asyncio.get_event_loop()
        cls.loop = urwid.MainLoop(ui, palette,
                                  event_loop=urwid.AsyncioEventLoop(loop=evl),
                                  pop_ups=True,
                                  **cls.extra_opts)

    @classmethod
    def exit(cls, err=0):
        log.info("Stopping eventloop")
        raise urwid.ExitMainLoop()

    @classmethod
    def redraw_screen(cls):
        try:
            cls.loop.draw_screen()
        except AssertionError as e:
            log.exception("exception failure in redraw_screen")
            raise e

    @classmethod
    def set_alarm_in(cls, interval, cb):
        handle = cls.loop.set_alarm_in(interval, cb)
        cls.add_alarm(handle, str(uuid.uuid1()))
        return handle

    @classmethod
    def add_alarm(cls, handle, name):
        if name in cls.alarms:
            cls.loop.remove_alarm(cls.alarms[name])
        cls.alarms[name] = handle

    @classmethod
    def remove_alarm(cls, handle):
        return cls.loop.remove_alarm(handle)

    @classmethod
    def remove_alarms(cls):
        for alarm in cls.alarms.values():
            cls.loop.remove_alarm(alarm)
        cls.alarms = {}

    @classmethod
    def screen_size(cls):
        return cls.loop.screen_size

    @classmethod
    def run(cls):
        """ Run eventloop
        """
        try:
            cls.loop.run()
        except:
            log.exception("Exception in ev.run():")
            raise
        return

    @classmethod
    def rows(cls):
        """ Returns rows
        """
        return cls.extra_opts['screen'].get_cols_rows()[1]

    @classmethod
    def columns(cls):
        """ Returns columns
        """
        return cls.extra_opts['screen'].get_cols_rows()[0]



""" Re-usable input widgets
"""

from urwid import (Edit, IntEdit, RadioButton, WidgetWrap, Columns)
import logging
import re

log = logging.getLogger("widgets.input")


class StringEditor(WidgetWrap):
    """ Edit input class
    Initializes and Edit object and attachs its result
    to the `value` accessor.
    """

    def __init__(self, caption=None, default=None, **kwargs):
        if caption is None:
            caption = ""
        self._edit = Edit(caption=caption, **kwargs)
        if default is not None:
            self._edit.set_edit_text(default)
        self.error = None
        super().__init__(self._edit)

    def keypress(self, size, key):
        if self.error:
            self._edit.set_edit_text("")
            self.error = None
        return super().keypress(size, key)

    def set_error(self, msg):
        self.error = msg
        return self._edit.set_edit_text(msg)

    @property
    def value(self):
        if self._edit.get_edit_text() == "":
            return None
        return self._edit.get_edit_text()

    @value.setter  # NOQA
    def value(self, value):
        self._edit.set_edit_text(value)


class PasswordEditor(StringEditor):
    """ Password input prompt with masking
    """

    def __init__(self, caption=None, mask="*"):
        super().__init__(caption, mask=mask)


class RealnameEditor(StringEditor):
    """ Username input prompt with input rules
    """

    def keypress(self, size, key):
        ''' restrict what chars we allow for username '''

        realname = r'[a-zA-Z0-9_\- ]'
        if re.match(realname, key) is None:
            return False

        return super().keypress(size, key)


class UsernameEditor(StringEditor):
    """ Username input prompt with input rules
    """

    def keypress(self, size, key):
        ''' restrict what chars we allow for username '''

        userlen = len(self.value)
        if userlen == 0:
            username = r'[a-z_]'
        else:
            username = r'[a-z0-9_-]'

        # don't allow non username chars
        if re.match(username, key) is None:
            return False

        return super().keypress(size, key)


class MountEditor(StringEditor):
    """ Mountpoint input prompt with input rules
    """

    def keypress(self, size, key):
        ''' restrict what chars we allow for mountpoints '''

        mountpoint = r'[a-zA-Z0-9_/\.\-]'
        if re.match(mountpoint, key) is None:
            return False

        return super().keypress(size, key)


class IntegerEditor(WidgetWrap):
    """ IntEdit input class
    """

    def __init__(self, caption=None, default=0):
        if caption is None:
            caption = ""
        self._edit = IntEdit(caption=caption, default=default)
        super().__init__(self._edit)

    @property
    def value(self):
        return self._edit.get_edit_text()


class Selector(WidgetWrap):
    """ Radio selection list of options
    """

    def __init__(self, opts):
        """
        :param list opts: list of options to display
        """
        self.opts = opts
        self.group = []
        self._add_options()
        super().__init__(Columns(self._add_options()))

    def _add_options(self):
        cols = []
        for item in self.opts:
            cols.append((8, RadioButton(self.group, item)))
        return cols

    @property
    def value(self):
        """ Returns value of selection.

        If the value is Yes/No will return True/False
        Any other values return the label.
        """
        for item in self.group:
            log.debug(item)
            if item.get_state() and item.label == "Yes":
                return True
            if item.get_state() and item.label == "No":
                return False
            if item.get_state():
                return item.label
        return None

    def set_default(self, item, state=False):
        """ Sets the default state of an item

        Arguments:
        item: name of item to change state of
        state: sets item to True or False
        """
        for i in self.group:
            if i.label == item:
                i.set_state(state)


class YesNo(Selector):
    """ Yes/No selector
    """

    def __init__(self):
        opts = ['Yes', 'No']
        super().__init__(opts)



from .infodialog import InfoDialogWidget  # noqa
from .selectordialog import (SelectorWidget,  # noqa
                             SelectorWithDescriptionWidget)  # noqa
from .error import ErrorView  # noqa



import urwid


# class PopUpDialog(urwid.WidgetWrap):
#     """A dialog that appears with nothing but a close button """
#     signals = ['close']
#     def __init__(self):
#         close_button = urwid.Button("that's pretty cool")
#         urwid.connect_signal(close_button, 'click',
#             lambda button:self._emit("close"))
#         pile = urwid.Pile([urwid.Text(
#             "^^  I'm attached to the widget that opened me. "
#             "Try resizing the window!\n"), close_button])
#         fill = urwid.Filler(pile)
#         self.__super.__init__(urwid.AttrWrap(fill, 'popbg'))


class DropDown(urwid.PopUpLauncher):

    def __init__(self, view):
        self.view = view
        super().__init__(self.view)
        urwid.connect_signal(self.original_widget, 'click',
                             lambda button: self.create_pop_up())

    def create_pop_up(self):
        urwid.connect_signal(self.self_view(), 'close',
                             lambda button: self.close_pop_up())

    def get_pop_up_parameters(self):
        return {'left': 0, 'top': 1,
                'overlay_width': 32, 'overlay_height': 7}



""" Represents a Single Juju service UI widget
"""

from urwid import Text
from ubuntui.widgets.juju.unit import UnitWidget


class ServiceWidget:

    def __init__(self, name, service):
        """ Init

        Params:
        machine: Juju Service Class
        """
        self.Name = Text(name)
        self.Units = []
        if service['units']:
            for n, unit in service['units'].items():
                w = UnitWidget(n, unit)
                self.Units.append(w)



from urwid import (WidgetWrap, Button, LineBox, Pile, Text)


class InfoDialogWidget(WidgetWrap):

    """A widget that displays a message and a close button."""

    def __init__(self, message, close_func):
        self.close_func = close_func
        button = Button("Close", self.do_close)
        box = LineBox(Pile([Text(message),
                            button]),
                      title="Info")
        super().__init__(box)

    def do_close(self, sender):
        self.close_func(self)



""" Base Frame Widget """

from urwid import Frame as _Frame
from urwid import WidgetWrap
from .anchors import Header, Footer, Body


class Frame(WidgetWrap):

    def __init__(self, header=None, body=None, footer=None):
        self.header = header if header else Header("Welcome")
        self.body = body if body else Body()
        self.footer = footer if footer else Footer()
        self.frame = _Frame(self.body,
                            header=self.header,
                            footer=self.footer)
        super().__init__(self.frame)

    def set_header(self, title=None, excerpt=None):
        self.frame.header = Header(title, excerpt)

    def set_subheader(self, text=""):
        self.frame.header.subheader = text

    def set_footer(self, message, completion=0):
        self.frame.footer = Footer(message, completion)

    def set_body(self, widget):
        self.frame.body = widget



from urwid import ListBox, SimpleListWalker, WidgetWrap


class SimpleList(WidgetWrap):

    def __init__(self, contents, is_selectable=True):
        self.contents = contents
        self.is_selectable = is_selectable
        super().__init__(self._build_widget())

    def _build_widget(self):
        lw = SimpleListWalker([x for x in self.contents])

        return ListBox(lw)

    def selectable(self):
        return self.is_selectable



""" An exception View equipped with traceback,
log output, and where to file a bug.
"""

from urwid import (ExitMainLoop, Pile, Text, Filler, WidgetWrap, Divider)
from ubuntui.widgets.buttons import cancel_btn
from ubuntui.utils import Color, Padding
import sys


class ErrorViewException(Exception):
    "Problem in Error  View"


class ErrorView(WidgetWrap):

    def __init__(self, error):
        body = [
            Padding.center_60(
                Text("Oops, there was a problem with your install:",
                     align="center")),
            Padding.center_95(
                Divider("\N{BOX DRAWINGS LIGHT HORIZONTAL}", 1, 1)),
            Padding.center_85(Text("Reason:")),
            Padding.center_80(Color.error_major(Text(error))),
            Padding.line_break(""),
            Padding.line_break(""),
            Padding.center_95(
                Divider("\N{BOX DRAWINGS LIGHT HORIZONTAL}", 1, 1)),
            Padding.center_20(self._build_buttons())
        ]
        super().__init__(Filler(Pile(body), valign="middle"))

    def _build_buttons(self):
        buttons = [
            Color.button_secondary(
                cancel_btn(label="Quit", on_press=self.cancel),
                focus_map="button_secondary focus")
        ]
        return Pile(buttons)

    def cancel(self, button):
        raise ExitMainLoop()



"""
urwid-ubuntu
============

Ubuntu styled widgets for urwid.
"""

from setuptools import setup, find_packages

setup(name='ubuntui',
      version="0.1.2",
      description="Urwid widgets used in Ubuntu console apps",
      long_description=__doc__,
      author="Canonical Solutions Engineering",
      author_email='ubuntu-dev@lists.ubuntu.com',
      url="https://github.com/Ubuntu-Solutions-Engineering/urwid-ubuntu",
      license="LGPL-3.0",
      packages=find_packages(exclude=['test']),
      install_requires=['urwid'])



from urwid import WidgetWrap, Text, Pile
from ubuntui.utils import Color


class StatusBarWidget(WidgetWrap):

    """ Displays text in the footers status area."""

    INFO = "[INFO]"
    ERROR = "[ERROR]"
    ARROW = " \u21e8 "

    def __init__(self, text=''):
        self._pending_deploys = Text('')
        self._status_line = Text(text, align="center")
        self._status_extra = self._build_status_extra()
        status = Pile([self._pending_deploys,
                       self._status_extra])
        super().__init__(status)

    def _build_status_extra(self):
        return Color.frame_footer(
            Pile([
                self._status_line
            ]))

    def message(self, text):
        """Write `text` on the footer."""
        self._status_line.set_text(text)

    def error_message(self, text):
        self.message([('status_error', self.ERROR),
                      self.ARROW + text])

    def info_message(self, text):
        self.message([('status_info', self.INFO),
                      self.ARROW + text])

    def set_pending_deploys(self, pending_deploys):
        if len(pending_deploys) > 0:
            msg = "Pending deploys: " + ", ".join(pending_deploys)
            self._pending_deploys.set_text(msg)
        else:
            self._pending_deploys.set_text('')

    def clear(self):
        """Clear the text."""
        self._w.set_text('')



from __future__ import unicode_literals
from .widgets.input import (StringEditor, YesNo, Selector, PasswordEditor)
from .utils import Color, Padding
from collections import OrderedDict
from urwid import (Pile, WidgetWrap, Text,
                   Button, Filler, Columns, Divider,
                   signals, emit_signal, connect_signal)

""" re-usable dialog widget """


def opts_to_ui(opts):
    """ Converts option dictionary suitable for input
    """
    converted_opts = []
    for k, v in opts.items():
        caption = k.replace('-', ' ')
        caption = "{} ".format(caption)
        if isinstance(v, bool):
            widget = YesNo()
        elif isinstance(v, list):
            widget = Selector(v)
        elif 'password' in k:
            widget = PasswordEditor()
        else:
            widget = StringEditor()
        converted_opts.append((k, caption, widget))
    return converted_opts


class Dialog(WidgetWrap):

    __metaclass__ = signals.MetaSignals
    signals = ['done']
    # key_conversion_map = {'tab': 'down', 'shift tab': 'up'}

    # List of Tuples in the form of ('key', 'caption', widget)
    input_items = []

    def __init__(self, title, cb):
        self.title = title
        self.cb = cb
        self.input_selection = OrderedDict()
        connect_signal(self, 'done', self.cb)
        super().__init__(self._build_widget())

    def _build_buttons(self):
        buttons = [
            Padding.line_break(""),
            Color.button_primary(
                Button("Confirm", self.submit),
                focus_map='button_primary focus'),
            Color.button_secondary(
                Button("Cancel", self.cancel),
                focus_map='button_secondary focus'),
        ]
        return Pile(buttons)

    def _build_widget(self, **kwargs):
        total_items = [
            Padding.center_60(Text(self.title, align="center")),
            Padding.center_60(
                Divider("\N{BOX DRAWINGS LIGHT HORIZONTAL}", 1, 1))
        ]
        if self.input_items:
            for item in self.input_items:
                key, caption, w = item
                self.input_selection[key] = w
                col = Columns(
                    [
                        ("weight", 0.5, Text(caption, align="right")),
                        Color.string_input(self.input_selection[key],
                                           focus_map="string_input focus")
                    ]
                )
                total_items.append(Padding.center_60(col))
        total_items.append(
            Padding.center_60(
                Divider("\N{BOX DRAWINGS LIGHT HORIZONTAL}", 1, 1)))
        total_items.append(Padding.center_20(self._build_buttons()))
        return Filler(Pile(total_items), valign='middle')

    def submit(self, button):
        self.emit_done_signal(self.input_selection)

    def cancel(self, button):
        raise SystemExit("Installation cancelled.")

    def emit_done_signal(self, *args):
        emit_signal(self, 'done', *args)



from __future__ import unicode_literals

from functools import partialmethod

from urwid import Padding as _Padding
from urwid import AttrMap, Divider, Text

from .palette import STYLES


def apply_padders(cls):
    """ Decorator for generating useful padding methods
    Loops through and generates methods like:
      Padding.push_1(Widget)
      Sets the left padding attribute by 1
      Padding.pull_24(Widget)
      Sets right padding attribute by 24.
      Padding.center_50(Widget)
      Provides center padding with a relative width of 50
    """
    padding_count = 100

    for i in range(1, padding_count):
        setattr(cls, 'push_{}'.format(i), partialmethod(_Padding, left=i))
        setattr(cls, 'pull_{}'.format(i), partialmethod(_Padding, right=i))
        setattr(cls, 'center_{}'.format(i),
                partialmethod(_Padding, align='center',
                              width=('relative', i)))
        setattr(cls, 'left_{}'.format(i),
                partialmethod(_Padding, align='left',
                              width=('relative', i)))
        setattr(cls, 'right_{}'.format(i),
                partialmethod(_Padding, align='right',
                              width=('relative', i)))
    return cls


@apply_padders
class Padding:
    """ Padding methods
    .. py:meth:: push_X(:class:`urwid.Widget`)
       This method supports padding the left side of the widget
       from 1-99, for example:
       .. code::
          Padding.push_20(Text("This will be indented 20 columns")
    .. py:meth:: pull_X(:class:`urwid.Widget`)
       This method supports padding the right side of the widget
       from 1-99, for example:
       .. code::
          Padding.pull_20(Text("This will be right indented 20 columns")
    .. py:meth:: center_X(:class:`urwid.Widget`)
       This method centers a widget with X being the relative width of
       the widget.
       .. code::
          Padding.center_10(Text("This will be centered with a "
                                 "width of 10 columns"))
    .. py:meth:: left_X(:class:`urwid.Widget`)
       This method aligns a widget left with X being the relative width of
       the widget.
       .. code::
          Padding.left_10(Text("This will be left aligned with a "
                               "width of 10 columns"))
    .. py:meth:: right_X(:class:`urwid.Widget`)
       This method right aligns a widget with X being the relative width of
       the widget.
       .. code::
          Padding.right_10(Text("This will be right aligned with a "
                                "width of 10 columns"))
    """
    line_break = partialmethod(Text)
    hr = partialmethod(Divider, "\N{BOX DRAWINGS LIGHT HORIZONTAL}", 1, 1)
    center = partialmethod(_Padding, align='center')
    left = partialmethod(_Padding, align='left')
    right = partialmethod(_Padding, align='right')


def apply_style_map(cls):
    """ Applies AttrMap attributes to Color class
    Eg:
      Color.frame_header(Text("I'm text in the Orange frame header"))
      Color.body(Text("Im text in wrapped with the body color"))
    """
    for k in STYLES:
        setattr(cls, k[0], partialmethod(AttrMap, attr_map=k[0]))
    return cls


@apply_style_map
class Color:
    """ Partial methods for :class:`~cloudinstall.ui.palette.STYLES`
    .. py:meth:: frame_header(:class:`urwid.Widget`)
       This method colors widget based on the style map used.
       .. code::
          Color.frame_header(Text("This will use foreground and background "
                                  "defined from the STYLES attribute"))
    """
    pass



""" Represents a Single Juju unit UI widget
"""

from urwid import Text


class UnitWidget:

    def __init__(self, name, unit):
        """ Init

        Params:
        machine: Juju Unit Class
        """
        self._unit = unit
        self._name = name
        self.Name = Text(self._name)
        self.PublicAddress = Text(unit.get('public-address', ''))
        self.Machine = Text(unit.get('machine', ''))

        agent_status = unit.get('agent-status', {})
        if agent_status:
            self.AgentStatus = Text(agent_status.get('status', ''))
            self.AgentStatusInfo = Text(agent_status.get('info', ''))

        workload = unit.get('workload-status', {})
        if workload:
            self.WorkloadInfo = Text(workload.get('info', ''))
            self.WorkloadStatus = Text(workload.get('status', ''))
        self.Icon = Text("")
